-- Example of a 10-bit shift register 
-- Designed in CSD subject - EETAC - UPC using the FSM like stype 

LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;
USE  IEEE.STD_LOGIC_ARITH.all;
USE  IEEE.STD_LOGIC_UNSIGNED.all;

ENTITY shift_register_10bit IS
      Port (	
		CLK		: IN	STD_LOGIC;
		CD 	  	: IN	STD_LOGIC;
		S 	  	: IN	STD_LOGIC_VECTOR(1 DOWNTO 0);
		RSI 	: IN	STD_LOGIC;
		LSI	  	: IN	STD_LOGIC;
		Q 	  	: OUT	STD_LOGIC_VECTOR(9 DOWNTO 0);
		D_IN  	: IN STD_LOGIC_VECTOR(9 DOWNTO 0)
	);

END shift_register_10bit;

			-- Internal desciption in FSM style

ARCHITECTURE FSM_like OF shift_register_10bit IS


CONSTANT Reset		: STD_LOGIC_VECTOR(9 DOWNTO 0) := "0000000000"; 

-- Internal wires --> in this case just the present and future state signals
SIGNAL present_state,future_state: STD_LOGIC_VECTOR(9 DOWNTO 0);

BEGIN
------------------------- State register 
-- The only clocked block, which is essentially a set of D-type flip-flops in parellel
-- The asynchronous reset has precedence over the CLK
State_Register: PROCESS (CD, CLK)
	BEGIN
		IF CD = '1' THEN 			-- reset counter ( an asynchronous reset which we call "Clear Direct"
			present_state <= Reset; 
			ELSIF (CLK'EVENT and CLK = '1') THEN	-- Synchronous register (D-type flip-flop)
				present_state <= future_state; 
		END IF;

END PROCESS State_Register;

-- CC1 ----Combinational circuit for calculating next state, which is setting the "future_state" signal
--         Generally, even for a simple FSM, this circuit will be complicated enough 
--         for using a process in order to code it easier.

CC1: PROCESS (present_state, D_IN, S, RSI, LSI)
	BEGIN
	IF S = "11" THEN 			
		future_state <= D_IN;
		ELSIF S="01" THEN     -- This is right shift of the putput bits; Q(0), which is also the RSO is lost
			future_state(0) <= present_state(1);
			future_state(1) <= present_state(2);
			future_state(2) <= present_state(3);
			future_state(3) <= present_state(4);
			future_state(4) <= present_state(5);
			future_state(5) <= present_state(6);
			future_state(6) <= present_state(7);
			future_state(7) <= present_state(8);
			future_state(8) <= present_state(9);
			future_state(9) <= RSI;
			
			ELSIF S="10" THEN -- This is left shift of the output bits; Q(9), which is also the LSO is lost
				future_state(9) <= present_state(8);
				future_state(8) <= present_state(7);
				future_state(7) <= present_state(6);
				future_state(6) <= present_state(5);
				future_state(5) <= present_state(4);
				future_state(4) <= present_state(3);
				future_state(3) <= present_state(2);
				future_state(2) <= present_state(1);
				future_state(1) <= present_state(0);
				future_state(0) <= LSI;	
	  	
	  			ELSE 	
					future_state <= present_state;
	END IF;	

END PROCESS CC1;
  
-- CC2 ---- Combinational circuit for calculating the outputs 
Q <= present_state; 

END FSM_like;